/*
 * Copyright (c) 2008, Laurent Desnogues <laurent.desnogues@gmail.com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the <organization> nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY <copyright holder> ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL <copyright holder> BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <stdint.h>

/* NOTE this should >= MAX_OUTS defined in generator.c */
uint32_t result_buffer[17];

/* APSR definitions */
#define F_N  (1u << 31)
#define F_Z  (1u << 30)
#define F_C  (1u << 29)
#define F_V  (1u << 28)
#define F_Q  (1u << 27)
#define F_G3 (1u << 19)
#define F_G2 (1u << 18)
#define F_G1 (1u << 17)
#define F_G0 (1u << 16)

#define DUMP_F(f,c) fputc(apsr & f ? c : '_', stdout)
static void dump_res(const char *str, unsigned n, int has_apsr)
{
  unsigned i;

  fputs(str, stdout);
  for (i = 0; i < n; i++)
    // TODO consider using putc to reduce library dependency
    printf("%08x ", result_buffer[i]);
  if (has_apsr != 0) {
    uint32_t apsr = result_buffer[n];

    DUMP_F(F_N , 'N');
    DUMP_F(F_Z , 'Z');
    DUMP_F(F_C , 'C');
    DUMP_F(F_V , 'V');
    DUMP_F(F_Q , 'Q');
    DUMP_F(F_G3, '3');
    DUMP_F(F_G2, '2');
    DUMP_F(F_G1, '1');
    DUMP_F(F_G0, '0');
  }
  fputc('\n', stdout);
}
#undef DUMP_F

static void run(void)
{
#include "test_arm.c"
}

int main(int argc, const char *argv[])
{
  run();

  return 0;
}
